export const getRadioValue = (name, element) => {
  let result = '';

  if (name && name !== '') {
    return name;
  }

  if ((name === '' && element.checked) || (!name && element.checked)) {
    result = element.value;
  }

  return result;
};

export const validateEmail = email => {
  const re = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  return re.test(String(email).toLowerCase());
};

export const validatePhone = number => {
  const result =
    number.length <= 9 ||
    (number.substr(0, 4) !== '+393' && parseInt(number.substr(0, 1)) !== 3);
  return !result;
};

export const validateCap = number => {
  const result = number.length >= 5;
  return result;
};

export const validatePassword = (array, minLength) => {
  let result = true;

  array.map((password, index) => {
    if (password.length < minLength) {
      result = false;
    }

    if (index === 1 && array[index - 1] !== password) {
      result = false;
    }
  });

  return result;
};

export const formToJSON = form => {
  const obj = {};
  const elements = form.querySelectorAll('input, select, textarea');

  Object.keys(elements).map(x => {
    const element = elements[x];
    const name = element.name;
    const value = element.value;

    if (name) {
      switch (element.type) {
        case 'textarea':
          obj[name] = value;
          break;
        case 'email':
          obj[name] = value;
          break;
        case 'checkbox':
          obj[name] = element.checked ? value : '';
          break;
        case 'radio':
          obj[name] = getRadioValue(obj[name], element);
          break;
        default:
          obj[name] = value;
      }
    }
  });

  return obj;
};

export const validateForm = (form, withErrors) => {
  const elements = form.querySelectorAll('input, select, textarea');
  const data = formToJSON(form);
  const result = {
    data,
    errors: {}
  };

  const passwordArray = [];

  Object.keys(elements).map(x => {
    const element = elements[x];
    const { name, required, type, value, minLength = '4' } = element;

    switch (type) {
      case 'email':
        if (required && !validateEmail(value)) {
          result.errors[name] = 'required';
        }
        break;
      case 'password':
        passwordArray.push(value);
        if (required && !validatePassword(passwordArray, minLength)) {
          result.errors[name] = 'required';
        }
        break;
      case 'tel':
        if (required && data[name] === '') {
          result.errors[name] = 'required';
        }

        if (
          required &&
          element.dataset.validate === 'tel' &&
          !validatePhone(value)
        ) {
          result.errors[name] = 'required';
        }

        if (
          required &&
          element.dataset.validate === 'cap' &&
          !validateCap(value)
        ) {
          result.errors[name] = 'required';
        }
        break;
      default:
        if (required && data[name] === '') {
          result.errors[name] = 'required';
        }
    }
  });

  if (withErrors) {
    addErrorsToForm(result.errors, form);
  }

  return result;
};

export const addErrorsToForm = (
  errors,
  form,
  blockSelector = '.js-form-input',
  errorClassName = 'has-error'
) => {
  Object.keys(errors).map(name => {
    const input = form.querySelector(`[name="${name}"]`);
    const parent = input.closest(blockSelector);
    parent.classList.add(errorClassName);
  });
};

export const addErrorsMessagesToForm = (
  messages,
  form,
  blockSelector = '.js-form-input',
  errorSelector = '.js-form-error'
) => {
  Object.keys(messages).map(name => {
    const input = form.querySelector(`[name="${name}"]`);
    const parent = input.closest(blockSelector);
    const error = parent.querySelector(errorSelector);
    const message = error.dataset.customError || messages[name];

    error.innerHTML = message;
  });
};

export const removeFormErrors = ({
  form,
  blockSelector = '.js-form-input',
  errorClassName = 'has-error',
  handler = () => {}
}) => {
  const elements = form.querySelectorAll('input, select, textarea');
  Object.keys(elements).map(x => {
    const input = elements[x];
    const parent = input.closest(blockSelector);
    input.addEventListener('focus', () => {
      parent.classList.remove(errorClassName);
      handler();
    });
    input.addEventListener('input', () => {
      parent.classList.remove(errorClassName);
      handler();
    });
    input.addEventListener('change', () => {
      parent.classList.remove(errorClassName);
      handler();
    });
  });
};

export const watchPhoneChange = form => {
  const validate = (input, limit = 12) => {
    let value = input.value;
    let maxLength = limit;

    if (value.substr(0, 1) === '+') {
      value = value.substr(1, value.length);
      value = value
        .replace(/\s+/g, '-')
        .replace(/[^0-9-]/g, '')
        .replace(/-+/g, '');
      value = '+' + value.substr(0, value.length);
      maxLength = limit + 3;
    } else {
      value = value
        .replace(/\s+/g, '-')
        .replace(/[^0-9-]/g, '')
        .replace(/-+/g, '');
      value = value.substr(0, value.length);
    }

    value = value.length <= maxLength ? value : value.substr(0, maxLength);

    input.value = value;
    input.initialValue = value;
  };

  const elements = form.querySelectorAll('[type="tel"]');

  Object.keys(elements).map(x => {
    const input = elements[x];
    input.addEventListener('input', () => {
      validate(input);
    });
  });
};

export const getAge = date => {
  const birthday = new Date(date);
  const today = new Date();

  let years = today.getFullYear() - birthday.getFullYear();

  if (
    today.getMonth() < birthday.getMonth() ||
    (today.getMonth() == birthday.getMonth() &&
      today.getDate() < birthday.getDate())
  ) {
    years--;
  }

  return years;
};
